﻿using UnityEngine;
using System.Collections;

public class EnemyController : MonoBehaviour {

	// Outlets
	public GameObject corpsePrefab;
	Transform transformComponent;
	SpriteRenderer spriteComponent;

	// Configuration
	public float healthMax;
	public float speed;

	// State Tracking
	public float healthCurrent;
	bool isDead;



	void OnDestroy() {
		GameController.instance.enemies.Remove(this);
	}

	void Awake() {
		transformComponent = GetComponent<Transform>();
		spriteComponent = GetComponent<SpriteRenderer>();
	}

	void Start() {
		healthCurrent = healthMax;
	}

	void Update() {
		if(isDead) {
			return;
		}

		if(GameController.instance.isGameOver) {
			transformComponent.Translate(-speed * 10f * Time.deltaTime, 0, 0);
		} else {
			transformComponent.Translate(-speed * Time.deltaTime, 0, 0);
		}
	}

	public void Init() {
		float speedModifier = Random.Range(-0.5f, 0.75f);
		speed = GameController.instance.enemySpeed + speedModifier;

		if(speedModifier > 0.5f) {
			GetComponent<Animator>().SetBool("isAngry", true);
		}

		float yShift = Random.Range(0, 0.75f);
		transformComponent.localPosition = new Vector3(transformComponent.localPosition.x, transformComponent.localPosition.y + yShift, transformComponent.localPosition.y + yShift);

		float randomScale = Random.Range(0.70f, 1f);
		transformComponent.localScale = new Vector3(randomScale, randomScale, randomScale);
	}

	void OnTriggerEnter2D(Collider2D other) {
		if(GameController.instance.isGameOver) {
			return;
		}

		if(other.gameObject.tag == "Player") {
			GameController.instance.TakeDamage(10f);
			Destroy(gameObject);
		}
	}

	public void TakeDamage(float amount) {
		SoundManager.PlayOneShot(SoundManager.instance.audioAttack);
		SoundManager.PlayOneShot(SoundManager.instance.audioKill);
		healthCurrent -= amount;
		UpdateDisplay();
		if(healthCurrent <= 0) {
			GameObject corpse = Instantiate (corpsePrefab) as GameObject;
			corpse.transform.position = transformComponent.position;
			corpse.transform.localScale = transformComponent.localScale;
			GetComponent<Animator>().SetTrigger("die");
			isDead = true;
		} else {
			transformComponent.Translate(0.5f, 0, 0);
		}
	}

	public void FinishDeath() {
		Destroy(gameObject);
	}

	void UpdateDisplay() {
//		spriteComponent.color = new Color(spriteComponent.color.r, spriteComponent.color.g, spriteComponent.color.b, (healthCurrent / healthMax) * 0.5f + 0.5f);
	}
}
