﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.SceneManagement;

public class GameController : MonoBehaviour {

	public static GameController instance;

	// Outlets
	public CameraController cameraComponent;
	public RectTransform uiRitual;

	public GameObject ritualStepPrefab;
	public Transform ritualWrapperP1;
	public Transform ritualWrapperP2;

	public GameObject enemyPrefab;
	public Transform enemySpawnPoint;

	public GameObject enemyPhysicsPrefab;
	public Transform enemyPhysicsSpawnPoint;

	public GameObject[] heroGibsPrefabs;
	public Transform heroGibSpawnPoint;

	public GameObject uiGameOver;

	public Text uiComboP1;
	public Text uiComboP2;

	public Text uiLevel;
	public Text uiProgress;
	public Text uiPoints;
	public Slider uiSliderProgress;

	public PlayerController player1;
	public PlayerController player2;

	public GameObject uiDamage;
	public Sprite[] uiDamageSprites;

	public Animator deathsplosion;
	public Animator playerDamage;

	// Configuration
	public List<KeyCode> gameplayKeys;
	public float transformCamStart;
	public float transformCamEnd;
	public float transformRatioUI;

	// State Tracking
	public bool isGameOver;

	public float ritualRate;
	float ritualElapsed;

	public float healthMax;
	public float healthCurrent;

	public List<EnemyController> enemies;
	public float enemyRate;
	float enemyElapsed;
	public float enemySpeed;

	Vector3 targetPositionCam;
	Vector3 targetPositionUI;

	int _comboP1;
	int _comboP2;
	public int comboP1 {
		get {
			return _comboP1;
		}
		set {
			_comboP1 = value;
			UpdateDisplayCombo();
		}
	}
	public int comboP2 {
		get {
			return _comboP2;
		}
		set {
			_comboP2 = value;
			UpdateDisplayCombo();
		}
	}

	int _level;
	public int level {
		get {
			return _level;
		}
		set {
			_level = value;
			LevelUp();
		}
	}

	float levelProgressMax;
	float _levelProgress;
	public float levelProgress {
		get {
			return _levelProgress;
		}
		set {
			_levelProgress = value;
			uiSliderProgress.value = _levelProgress / levelProgressMax;
			if(_levelProgress >= levelProgressMax) {
				level++;
				levelProgress = 0;
			}
			uiProgress.text = _levelProgress + "/" + levelProgressMax;
		}
	}

	float _points;
	public float points {
		get {
			return _points;
		}
		set {
			_points = value;
			uiPoints.text = _points.ToString();
		}
	}

	float uiDamageTimer;



	void Awake() {
		instance = this;
	}

	void Start() {
		level = 0;
		points = 0;
		levelProgress = 0;
		healthCurrent = healthMax;

		gameplayKeys = new List<KeyCode> () {
			KeyCode.W, 
			KeyCode.A, 
			KeyCode.S, 
			KeyCode.D, 
			KeyCode.I, 
			KeyCode.J, 
			KeyCode.K, 
			KeyCode.L
		};

		enemies = new List<EnemyController>();

		UpdateDisplay();
	}

	void Update() {
		if(!isGameOver) {
			// Spawn Ritual Steps
			if(ritualRate > 0) {
				ritualElapsed += Time.deltaTime;
				if(ritualElapsed > ritualRate) {
					PrepRitual();
					ritualElapsed = 0;
				}
			}

			// Spawn Enemies
			enemyElapsed += Time.deltaTime;
			if(enemyElapsed > (enemyRate + Random.Range(-enemyRate / 10f, enemyRate / 10f))) {
				SpawnEnemy();
				enemyElapsed = 0;
			}
		}

		// Lerp Display
		cameraComponent.transform.localPosition = Vector3.Lerp(cameraComponent.transform.localPosition, targetPositionCam, 0.2f);
		uiRitual.anchoredPosition = Vector3.Lerp(uiRitual.anchoredPosition, targetPositionUI, 0.2f);

		if(Input.GetKeyDown(KeyCode.Space)) {
			TakeDamage(1000f);
		}
	}

	void PrepRitual() {
		GameObject newRitualStep = Instantiate(ritualStepPrefab) as GameObject;
		RitualStep ritualStepComponent = newRitualStep.GetComponent<RitualStep>();
		ritualStepComponent.Init(1, gameplayKeys[Random.Range(1, 4)]);
		newRitualStep.transform.SetParent(ritualWrapperP1);
		newRitualStep.transform.localScale = Vector3.one;
		newRitualStep.GetComponent<RectTransform>().anchoredPosition = new Vector3(35, 145, 0);

		newRitualStep = Instantiate(ritualStepPrefab) as GameObject;
		ritualStepComponent = newRitualStep.GetComponent<RitualStep>();
		ritualStepComponent.Init(2, gameplayKeys[Random.Range(5, 8)]);
		newRitualStep.transform.SetParent(ritualWrapperP2);
		newRitualStep.transform.localScale = Vector3.one;
		newRitualStep.GetComponent<RectTransform>().anchoredPosition = new Vector3(35, 145, 0);
	}

	void SpawnEnemy() {
		GameObject newEnemy = Instantiate(enemyPrefab) as GameObject;
		newEnemy.transform.position = enemySpawnPoint.position;

		EnemyController enemyComponent = newEnemy.GetComponent<EnemyController>();
		enemyComponent.Init();
		enemies.Add(enemyComponent);
	}

	public void TakeDamage(float amount) {
		playerDamage.SetTrigger("damage");
		SoundManager.PlayOneShot(SoundManager.instance.audioHurt);
		if(healthCurrent > 0) {
			healthCurrent -= amount;
			UpdateDisplay();
			if(healthCurrent <= 0) {
				GameOver();
			} else {
				uiDamageTimer = 0;
				uiDamage.GetComponent<Image>().sprite = uiDamageSprites[Random.Range(0, uiDamageSprites.Length)];
				StartCoroutine(DamageHandler());
			}
		}
	}

	IEnumerator DamageHandler() {
		yield return null;

		// Set Color based on Health and transition alpha
		float alpha = (0.5f - uiDamageTimer) / 2f;
		Color targetColor = new Color(1f, healthCurrent / healthMax - 0.2f, healthCurrent / healthMax - 0.2f, alpha);

		// Transition Transparency
		uiDamage.GetComponent<Image>().color = targetColor;

		// Turn on UI
		uiDamage.GetComponent<Image>().enabled = true;

		// Track Time
		uiDamageTimer += Time.deltaTime;

		// Repeat
		if(uiDamageTimer < 0.25f) {
			StartCoroutine(DamageHandler());
		} else {
			uiDamage.GetComponent<Image>().enabled = false;
		}
	}

	void UpdateDisplay() {
		float healthRatio = healthCurrent / healthMax;
		float cameraPosition = healthRatio * (transformCamStart - transformCamEnd) + transformCamEnd;
		targetPositionCam = new Vector3(cameraPosition, 0, -10f);
		targetPositionUI = new Vector3(cameraPosition * transformRatioUI, 0, 0);
	}

	public void UpdateDisplayCombo() {
		// Player 1
		if(comboP1 > 1) {
			string suffix = "";

			int exclamationCount = comboP1 / 5;
			for(int i = 0; i < exclamationCount; i++) {
				suffix += "!";
			}

			uiComboP1.text = comboP1 + suffix;
		} else {
			uiComboP1.text = "";
		}

		// Player 2
		if(comboP2 > 1) {
			string suffix = "";

			int exclamationCount = comboP2 / 5;
			for(int i = 0; i < exclamationCount; i++) {
				suffix += "!";
			}

			uiComboP2.text = comboP2 + suffix;
		} else {
			uiComboP2.text = "";
		}
	}

	public void DoDamage(int player) {
		// Level Up Progress
		float comboProgress = 0;
		if(player == 1) {
			comboProgress = (float)comboP1;
		} else if(player == 2) {
			comboProgress = (float)comboP2;
		}
		levelProgress += 1f + comboProgress;
		points += 1f + comboProgress;

		// Do Attack
		bool canAttack = false;
		if(player == 1 && comboP1 > 0 && comboP1 % 3 == 0) {
			canAttack = true;
		} else if(player == 2 && comboP2 > 0 && comboP2 % 3 == 0) {
			canAttack = true;
		}
		if(enemies.Count > 0 && canAttack) {
			if(player == 1) {
				player1.Attack();
			} else if(player == 2) {
				player2.Attack();
			}
		}
	}

	public void DoDamageHandler() {
		EnemyController targetEnemy = GetClosestEnemy();
		if(enemies.Count > 0 && targetEnemy != null) {
			targetEnemy.TakeDamage(100f);
		}
	}

	void GameOver() {
		isGameOver = true;
		ritualRate = 0;
		healthCurrent = -50f;
		UpdateDisplay();
		StartCoroutine("GameOverHandler");
		StartCoroutine("GameOverHandler2");
		SoundManager.instance.backgroundSource.Stop();
	}

	public void TryAgain() {
		SceneManager.LoadScene(SceneManager.GetActiveScene().buildIndex);
	}

	IEnumerator GameOverHandler() {
		SpawnPhysicsEnemy();
		yield return new WaitForSeconds(0.01f);
		StartCoroutine("GameOverHandler");
	}

	void SpawnPhysicsEnemy() {
		GameObject newEnemy = Instantiate(enemyPhysicsPrefab) as GameObject;
		newEnemy.transform.position = enemyPhysicsSpawnPoint.position;
		newEnemy.transform.localPosition = new Vector3(newEnemy.transform.localPosition.x, newEnemy.transform.localPosition.y, 0);

		Rigidbody2D rigidbodyComponent = newEnemy.GetComponent<Rigidbody2D>();
		rigidbodyComponent.AddForce(new Vector2(Random.Range(-750f, -500f), Random.Range(-200f, 500f)));

		float randomScale = Random.Range(0.70f, 1f);
		newEnemy.transform.localScale = new Vector3(randomScale, randomScale, randomScale);
	}

	IEnumerator GameOverHandler2() {
		SoundManager.PlayOneShot(SoundManager.instance.audioGameOver);
		yield return new WaitForSeconds(0.5f);
		SoundManager.PlayOneShot(SoundManager.instance.audioMaul);
		yield return new WaitForSeconds(0.5f);
		SoundManager.PlayOneShot(SoundManager.instance.audioHiss);
		yield return new WaitForSeconds(1f);
		SoundManager.PlayOneShot(SoundManager.instance.audioScream);
		yield return new WaitForSeconds(1f);
		SoundManager.PlayOneShot(SoundManager.instance.audioGibs);

		deathsplosion.SetTrigger("explode");
		for(int j = 0; j < 5; j++) {
			for(int i = 0; i < heroGibsPrefabs.Length; i++) {
				GameObject heroGibPrefab = heroGibsPrefabs[i];
				GameObject newGib = Instantiate(heroGibPrefab) as GameObject;
				newGib.transform.position = heroGibSpawnPoint.position;
				newGib.transform.localPosition = new Vector3(newGib.transform.localPosition.x, newGib.transform.localPosition.y, 0);

				Rigidbody2D rigidbodyComponent = newGib.GetComponent<Rigidbody2D>();
				rigidbodyComponent.AddForce(new Vector2(Random.Range(500f, 2000f), Random.Range(-200f, 500f)));
				rigidbodyComponent.AddTorque(Random.Range(-1000f, 1000f));
			}
		}

		yield return new WaitForSeconds(2f);
		uiGameOver.SetActive(true);
		SoundManager.PlayOneShot(SoundManager.instance.audioTryAgain);
	}

	void LevelUp() {
		// UI
		uiLevel.text = "Level " + (level + 1);
		if(level > 0) {
			SoundManager.PlayOneShot(SoundManager.instance.audioLevelUp);
		}

		// Level Requirements
		levelProgressMax = 50f + (float)level * 50f;

		// Enemy Rate
		enemyRate = 3f - (float)level * 0.25f;

		// Enemy Speed
		enemySpeed = 1f + (float)level * 0.1f;

		// Ritual Rate
		ritualRate = 1f - (float)level * 0.1f;
	}

	EnemyController GetClosestEnemy() {
		EnemyController closestEnemy = null;
		foreach(EnemyController enemy in enemies) {
			if(closestEnemy == null) {
				closestEnemy = enemy;
			} else {
				if(enemy.transform.localPosition.x < closestEnemy.transform.localPosition.x) {
					closestEnemy = enemy;
				}
			}
		}

		return closestEnemy;
	}
}
