﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class RitualStep : MonoBehaviour {

	// Outlets
	public Sprite[] backgrounds;
	public Text label;
	public Image background;
	Transform transformComponent;
	RectTransform rectTransformComponent;

	// Configuration
	int player;
	public KeyCode targetKey;
	float thresholdSuccess = 55f;
	float thresholdFail = 15f;

	// State Tracking
	public bool isSuccessful;
	bool isFailed;
	bool isActive;
	bool isDisappearing;



	void Start() {
		transformComponent = GetComponent<Transform>();
		rectTransformComponent = GetComponent<RectTransform>();
		background.sprite = backgrounds[player - 1];
	}

	void Update() {
		if(GameController.instance.isGameOver) {
			Destroy(gameObject);
			return;
		}

		// Move Downward
		transformComponent.Translate(new Vector3(0, -(60f / GameController.instance.ritualRate) * Time.deltaTime, 0));

		// Handle Progress
		if(rectTransformComponent.anchoredPosition.y <= thresholdSuccess && rectTransformComponent.anchoredPosition.y > thresholdFail) {
			// Go Active!
			if(!isActive) {
				background.color = Color.white;
				isActive = true;
			}

			// Check for SUCCESS
			if(Input.GetKeyDown(targetKey)) {
				Success();
			}
		} else if(rectTransformComponent.anchoredPosition.y <= thresholdFail) {
			// FAIL and/or DISAPPEAR
			if(!isSuccessful) {
				Fail();
			}
			Disappear();
		}
	}

	public void Init(int aPlayer, KeyCode aTargetKey) {
		player = aPlayer;
		targetKey = aTargetKey;
		UpdateDisplay();
	}

	public void UpdateDisplay() {
		label.text = targetKey.ToString();
	}

	void Success() {
		if(!isSuccessful) {
			SoundManager.PlayOneShot(SoundManager.instance.audioSuccess);
			if(player == 1) {
				GameController.instance.comboP1++;
			} else if(player == 2) {
				GameController.instance.comboP2++;
			}
			GameController.instance.DoDamage(player);
		}
		isSuccessful = true;
		background.color = new Color(0, 1f, 0, background.color.a);
	}

	void Fail() {
		if(!isFailed) {
			background.color = new Color(1f, 0, 0, background.color.a);
			if(player == 1) {
				GameController.instance.comboP1 = 0;
			} else if(player == 2) {
				GameController.instance.comboP2 = 0;
			}

			isFailed = true;
		}
	}

	void Disappear() {
		if(!isDisappearing) {
			StartCoroutine(DisappearHandler());
		}
		isDisappearing = true;
	}

	IEnumerator DisappearHandler() {
		background.color = new Color(background.color.r, background.color.g, background.color.b, background.color.a - 0.1f);
		yield return new WaitForSeconds(0.05f);

		if(background.color.a < 0) {
			Destroy(gameObject);
		} else {
			StartCoroutine(DisappearHandler());
		}
	}
}
